@@ -65,9 +65,13 @@ class imageDisk {
 		virtual void Get_Geometry(uint32_t * getHeads, uint32_t *getCyl, uint32_t *getSect, uint32_t *getSectSize);
 		virtual uint8_t GetBiosType(void);
 		virtual uint32_t getSectSize(void);
+		imageDisk(class DOS_Drive *useDrive, unsigned int letter, int freeMB, int timeout);
 		imageDisk(FILE *imgFile, const char *imgName, uint32_t imgSizeK, bool isHardDisk);
 		imageDisk(FILE* diskimg, const char* diskName, uint32_t cylinders, uint32_t heads, uint32_t sectors, uint32_t sector_size, bool hardDrive);
-		virtual ~imageDisk() { if(diskimg != NULL) { fclose(diskimg); diskimg=NULL; } };
+		virtual ~imageDisk();
+		void Set_GeometryForHardDisk();
+		struct fatFromDOSDrive* ffdd = NULL;
+		unsigned int drvnum = DOS_DRIVES;
 
 		IMAGE_TYPE class_id = ID_BASE;
 		std::string diskname;
@@ -91,6 +95,7 @@ class imageDisk {
 	private:
 		volatile int refcount = 0;
 		std::vector<bool> partition_in_use; /* used by FAT driver to prevent mounting a partition twice */
+		uint64_t current_fpos;
 
 	public:
 		int Addref() {
@@ -397,7 +402,7 @@ public:
         (void)data;//UNUSED
         return 0x05; /* fail, read only */
     }
-    imageDiskElToritoFloppy(unsigned char new_CDROM_drive,unsigned long new_cdrom_sector_offset,unsigned char floppy_emu_type) : imageDisk(NULL,NULL,0,false) {
+    imageDiskElToritoFloppy(unsigned char new_CDROM_drive,unsigned long new_cdrom_sector_offset,unsigned char floppy_emu_type) : imageDisk((FILE *)NULL,NULL,0,false) {
         diskimg = NULL;
         sector_size = 512;
         CDROM_drive = new_CDROM_drive;
