@@ -29,7 +29,7 @@ enum { CB_RETN,CB_RETF,CB_RETF8,CB_IRET,CB_IRETD,CB_IRET_STI,CB_IRET_EOI_PIC1,
 		CB_IRQ0,CB_IRQ1,CB_IRQ1_BREAK,CB_IRQ9,CB_IRQ12,CB_IRQ12_RET,CB_MOUSE,
 		/*CB_INT28,*/CB_INT29,CB_INT16,CB_HOOKABLE,CB_TDE_IRET,CB_IPXESR,CB_IPXESR_RET,
 		CB_INT21,CB_INT13,CB_VESA_WAIT,CB_VESA_PM,CB_IRET_EOI_PIC2,CB_CPM,
-        CB_RETF_STI,CB_RETF_CLI};
+        CB_RETF_STI,CB_RETF_CLI,CB_INT6F_ATOK};
 
 /* NTS: Cannot make runtime configurable, because CB_MAX is used to define an array */
 #define CB_MAX		128U
