@@ -107,7 +107,7 @@ public:
     void SwitchToSecureMode() { secure_mode = true; }//can't be undone
     void ClearExtraData() { Section_prop *sec_prop; Section_line *sec_line; for (const_it tel = sectionlist.begin(); tel != sectionlist.end(); ++tel) {sec_prop = dynamic_cast<Section_prop *>(*tel); sec_line = dynamic_cast<Section_line *>(*tel); if (sec_prop) sec_prop->data = ""; else if (sec_line) sec_line->data = "";} }
 public:
-    std::string opt_editconf,opt_opensaves,opt_opencaptures,opt_lang;
+    std::string opt_editconf,opt_opensaves,opt_opencaptures,opt_lang="",opt_machine="";
     std::vector<std::string> config_file_list;
     std::vector<std::string> opt_o;
     std::vector<std::string> opt_c;
