@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2002-2020  The DOSBox Team
+ *  Copyright (C) 2002-2021  The DOSBox Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -19,9 +19,7 @@
 #ifndef DOSBOX_MEM_H
 #define DOSBOX_MEM_H
 
-#ifndef DOSBOX_DOSBOX_H
 #include "dosbox.h"
-#endif
 
 #include "byteorder.h"
 
@@ -63,14 +61,38 @@ MemHandle                   MEM_NextHandleAt(MemHandle handle,Bitu where);
     Working on big or little endian machines 
 */
 
-#if !defined(C_UNALIGNED_MEMORY)
-/* meaning: we're probably being compiled for a processor that doesn't like unaligned WORD access,
-            on such processors typecasting memory as uint16_t and higher can cause a fault if the
-        address is not aligned to that datatype when we read/write through it. */
-
 static INLINE uint8_t host_readb(ConstHostPt const off) {
     return *off;
 }
+
+static INLINE void host_writeb(HostPt const off,const uint8_t val) {
+    *off = val;
+}
+
+// use __builtin_bswap* for gcc >= 4.3
+#if defined(WORDS_BIGENDIAN) && defined(__GNUC__) && \
+    (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 3))
+
+static INLINE uint16_t host_readw(ConstHostPt off) {
+    return __builtin_bswap16(*(uint16_t *)off);
+}
+static INLINE uint32_t host_readd(ConstHostPt off) {
+    return __builtin_bswap32(*(uint32_t *)off);
+}
+static INLINE void host_writew(HostPt const off, const uint16_t val) {
+    *(uint16_t *)off = __builtin_bswap16(val);
+}
+static INLINE void host_writed(HostPt const off, const uint32_t val) {
+    *(uint32_t *)off = __builtin_bswap32(val);
+}
+static INLINE void host_writeq(HostPt const off, const uint64_t val) {
+    *(uint64_t *)off = __builtin_bswap64(val);
+}
+#elif !defined(C_UNALIGNED_MEMORY)
+/* !defined(C_UNALIGNED_MEMORY) meaning: we're probably being compiled for a processor that doesn't like unaligned WORD access,
+        on such processors typecasting memory as uint16_t and higher can cause a fault if the
+        address is not aligned to that datatype when we read/write through it. */
+
 static INLINE uint16_t host_readw(ConstHostPt const off) {
     return (uint16_t)host_readb(off) + ((uint16_t)host_readb(off+(uintptr_t)1U) << (uint16_t)8U);
 }
@@ -81,9 +103,7 @@ static INLINE uint64_t host_readq(ConstHostPt const off) {
     return (uint64_t)host_readd(off) + ((uint64_t)host_readd(off+(uintptr_t)4U) << (uint64_t)32U);
 }
 
-static INLINE void host_writeb(HostPt const off,const uint8_t val) {
-    *off = val;
-}
+
 static INLINE void host_writew(HostPt const off,const uint16_t val) {
     host_writeb(off,   (uint8_t)(val));
     host_writeb(off+1U,(uint8_t)(val >> (uint16_t)8U));
@@ -99,9 +119,6 @@ static INLINE void host_writeq(HostPt const off,const uint64_t val) {
 
 #else
 
-static INLINE uint8_t host_readb(ConstHostPt const off) {
-    return *(const uint8_t *)off;
-}
 static INLINE uint16_t host_readw(ConstHostPt const off) {
     return le16toh((*(const uint16_t *)off)); // BSD endian.h
 }
@@ -112,9 +129,6 @@ static INLINE uint64_t host_readq(ConstHostPt const off) {
     return le64toh((*(const uint64_t *)off)); // BSD endian.h
 }
 
-static INLINE void host_writeb(HostPt const off,const uint8_t val) {
-    *(uint8_t *)(off) = val;
-}
 static INLINE void host_writew(HostPt const off,const uint16_t val) {
     *(uint16_t *)(off) = htole16(val);
 }
@@ -129,7 +143,7 @@ static INLINE void host_writeq(HostPt const off,const uint64_t val) {
 
 
 static INLINE void var_write(uint8_t * const var, const uint8_t val) {
-    host_writeb((HostPt)var, val);
+    host_writeb(var, val);
 }
 
 static INLINE void var_write(uint16_t * const var, const uint16_t val) {
@@ -144,6 +158,14 @@ static INLINE void var_write(uint64_t * const var, const uint64_t val) {
     host_writeq((HostPt)var, val);
 }
 
+static INLINE uint16_t var_read(uint16_t * var) {
+    return host_readw((ConstHostPt)var);
+}
+
+static INLINE uint32_t var_read(uint32_t * var) {
+    return host_readd((ConstHostPt)var);
+}
+
 /* The Folowing six functions are slower but they recognize the paged memory system */
 
 uint8_t  mem_readb(const PhysPt address);
@@ -181,7 +203,7 @@ static INLINE uint32_t phys_readd(const PhysPt addr) {
 
 /* These don't check for alignment, better be sure it's correct */
 
-void MEM_BlockWrite(PhysPt pt,void const * const data,Bitu size);
+void MEM_BlockWrite(PhysPt pt, const void *data, size_t size);
 void MEM_BlockRead(PhysPt pt,void * data,Bitu size);
 void MEM_BlockWrite32(PhysPt pt,void * data,Bitu size);
 void MEM_BlockRead32(PhysPt pt,void * data,Bitu size);
@@ -195,32 +217,32 @@ void mem_strcpy(PhysPt dest,PhysPt src);
 /* The folowing functions are all shortcuts to the above functions using physical addressing */
 
 static inline constexpr PhysPt PhysMake(const uint16_t seg,const uint16_t off) {
-    return ((PhysPt)seg << (PhysPt)4U) + (PhysPt)off;
+    return ((PhysPt)seg << 4U) + (PhysPt)off;
 }
 
 static inline constexpr uint16_t RealSeg(const RealPt pt) {
-    return (uint16_t)((RealPt)pt >> (RealPt)16U);
+    return (uint16_t)(pt >> 16U);
 }
 
 static inline constexpr uint16_t RealOff(const RealPt pt) {
-    return (uint16_t)((RealPt)pt & (RealPt)0xffffu);
+    return (uint16_t)(pt & 0xffffu);
 }
 
 static inline constexpr PhysPt Real2Phys(const RealPt pt) {
-    return (PhysPt)(((PhysPt)RealSeg(pt) << (PhysPt)4U) + (PhysPt)RealOff(pt));
+    return (((PhysPt)RealSeg(pt) << 4U) + (PhysPt)RealOff(pt));
 }
 
 static inline constexpr RealPt RealMake(const uint16_t seg,const uint16_t off) {
-    return (RealPt)(((RealPt)seg << (RealPt)16U) + (RealPt)off);
+    return (((RealPt)seg << 16U) + (RealPt)off);
 }
 
 /* convert physical address to 4:16 real pointer (example: 0xABCDE -> 0xA000:0xBCDE) */
 static inline constexpr RealPt PhysToReal416(const PhysPt phys) {
-    return RealMake((uint16_t)(((PhysPt)phys >> (PhysPt)4U) & (PhysPt)0xF000U),(uint16_t)((PhysPt)phys & (PhysPt)0xFFFFU));
+    return RealMake((uint16_t)((phys >> 4U) & 0xF000U),(uint16_t)(phys & 0xFFFFU));
 }
 
 static inline constexpr PhysPt RealVecAddress(const uint8_t vec) {
-    return (PhysPt)((unsigned int)vec << 2U);
+    return ((unsigned int)vec << 2U);
 }
 
 
@@ -250,7 +272,7 @@ static INLINE RealPt RealGetVec(const uint8_t vec) {
 }
 
 static INLINE void RealSetVec(const uint8_t vec,const RealPt pt) {
-    mem_writed(RealVecAddress(vec),(uint32_t)pt);
+    mem_writed(RealVecAddress(vec),pt);
 }
 
 static INLINE void RealSetVec(const uint8_t vec,const RealPt pt,RealPt &old) {
@@ -260,4 +282,3 @@ static INLINE void RealSetVec(const uint8_t vec,const RealPt pt,RealPt &old) {
 }
 
 #endif
-
