@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2002-2020  The DOSBox Team
+ *  Copyright (C) 2002-2021  The DOSBox Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -19,11 +19,7 @@
 #ifndef DOSBOX_REGS_H
 #define DOSBOX_REGS_H
 
-#include <iostream>
-
-#ifndef DOSBOX_MEM_H
 #include "mem.h"
-#endif
 
 #define FLAG_CF		0x00000001U
 #define FLAG_PF		0x00000004U
