@@ -20,7 +20,7 @@ if [ $? -eq 0 ]; then
 	echo  flatpak --user install myrepo com.dosbox_x.DOSBox-X
 	echo
 	echo You can then run the flatpak as follows:
-	echo  flatpak run com.dosbox_x.DOSBox-X
+	echo  flatpak --user run com.dosbox_x.DOSBox-X
 	echo
 	echo Or you can test it without installing by running:
 	echo  flatpak-builder --run build-flatpak contrib/linux/com.dosbox_x.DOSBox-X.yaml dosbox-x
