@@ -169,7 +169,7 @@ static struct {
 } decoder_pagefault;
 
 #if defined(X86_DYNFPU_DH_ENABLED)
-static struct dyn_dh_fpu {
+static struct dyn_dh_fpu_struct {
 	uint16_t	cw,host_cw;
 	bool		state_used;
 	// some fields expanded here for alignment purposes
@@ -335,7 +335,7 @@ restart_core:
 			while (decoder_pagefault.had_pagefault) {
 				// Can happen only if use_dynamic_core_with_paging is on
 				// We can't throw exception during the creation of the block, as it will corrupt things
-				// If a page fault occoured, invalidated the block, and try to create smaller block
+				// If a page fault occurred, invalidated the block, and try to create smaller block
 				// If the page fault is in the current instruction, throw exception
 				block->Clear();
 				if (cache_size == 1)
