@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2002-2020  The DOSBox Team
+ *  Copyright (C) 2002-2021  The DOSBox Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,17 +16,10 @@
  *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
-
-#include "dosbox.h"
+#include "config.h"
 
 #if (C_DYNREC)
-
-#include <assert.h>
-#include <stdarg.h>
-#include <stdio.h>
 #include <string.h>
-#include <stddef.h>
-#include <stdlib.h>
 
 #if defined (WIN32)
 #include <windows.h>
@@ -91,6 +84,8 @@
 
 // access to a general register
 #define DRCD_REG_VAL(reg) (&cpu_regs.regs[reg].dword)
+// access to the flags register
+#define DRCD_REG_FLAGS (&cpu_regs.flags)
 // access to a segment register
 #define DRCD_SEG_VAL(seg) (&Segs.val[seg])
 // access to the physical value of a segment register/selector
@@ -112,7 +107,8 @@ enum BlockReturn {
 #endif
 	BR_Iret,
 	BR_CallBack,
-	BR_SMCBlock
+	BR_SMCBlock,
+	BR_Trap
 };
 
 // identificator to signal self-modification of the currently executed block
@@ -263,6 +259,7 @@ Bits CPU_Core_Dynrec_Run(void) {
     }
 
 	for (;;) {
+		dosbox_allow_nonrecursive_page_fault = false;
 		// Determine the linear address of CS:EIP
 		PhysPt ip_point=SegPhys(cs)+reg_eip;
 		#if C_HEAVY_DEBUG
@@ -278,7 +275,10 @@ Bits CPU_Core_Dynrec_Run(void) {
 		}
 
 		// page doesn't contain code or is special
-		if (GCC_UNLIKELY(!chandler)) return CPU_Core_Normal_Run();
+		if (GCC_UNLIKELY(!chandler)) {
+			dosbox_allow_nonrecursive_page_fault = true;
+			return CPU_Core_Normal_Run();
+		}
 
 		// find correct Dynamic Block to run
 		CacheBlockDynRec * block=chandler->FindCacheBlock(ip_point&4095);
@@ -289,15 +289,17 @@ Bits CPU_Core_Dynrec_Run(void) {
 				// translate up to 32 instructions
 				block=CreateCacheBlock(chandler,ip_point,32);
 			} else {
+				dosbox_allow_nonrecursive_page_fault = true;
 				// let the normal core handle this instruction to avoid zero-sized blocks
 				cpu_cycles_count_t old_cycles=CPU_Cycles;
 				CPU_Cycles=1;
+				CPU_CycleLeft+=old_cycles;
 				Bits nc_retcode=CPU_Core_Normal_Run();
 				if (!nc_retcode) {
 					CPU_Cycles=old_cycles-1;
+					CPU_CycleLeft-=old_cycles;
 					continue;
 				}
-				CPU_CycleLeft+=old_cycles;
 				return nc_retcode;
 			}
 		}
@@ -306,7 +308,7 @@ run_block:
 		cache.block.running=0;
 		// now we're ready to run the dynamic code block
 //		BlockReturn ret=((BlockReturn (*)(void))(block->cache.start))();
-		BlockReturn ret=core_dynrec.runcode(block->cache.start);
+		BlockReturn ret=core_dynrec.runcode(block->cache.xstart);
 
         if (sizeof(CPU_Cycles) > 4) {
             // HACK: All dynrec cores for each processor assume CPU_Cycles is 32-bit wide.
@@ -356,7 +358,7 @@ run_block:
 			return CBRET_NONE;
 
 		case BR_CallBack:
-			// the callback code is executed in dosbox.conf, return the callback number
+			// the callback code is executed in dosbox-x.conf, return the callback number
 			FillFlags();
 			return (Bits)core_dynrec.callback;
 
@@ -373,6 +375,7 @@ run_block:
 			// handle this instruction
 			CPU_CycleLeft+=CPU_Cycles;
 			CPU_Cycles=1;
+			dosbox_allow_nonrecursive_page_fault = true;
 			return CPU_Core_Normal_Run();
 
 		case BR_Link1:
@@ -381,6 +384,18 @@ run_block:
 			if (block) goto run_block;
 			break;
 
+		case BR_Trap:
+			// trapflag is set, switch to the trap-aware decoder
+	#if C_DEBUG
+	#if C_HEAVY_DEBUG
+			if (DEBUG_HeavyIsBreakpoint()) {
+				return debugCallback;
+			}
+	#endif
+	#endif
+			cpudecoder=CPU_Core_Dynrec_Trap_Run;
+			return CBRET_NONE;
+
 		default:
 			E_Exit("Invalid return code %d", ret);
 		}
@@ -398,7 +413,7 @@ Bits CPU_Core_Dynrec_Trap_Run(void) {
 
 	// trap to int1 unless the last instruction deferred this
 	// (allows hardware interrupts to be served without interaction)
-	if (!cpu.trap_skip) CPU_HW_Interrupt(1);
+	if (!cpu.trap_skip) CPU_DebugException(DBINT_STEP,reg_eip);
 
 	CPU_Cycles = oldCycles-1;
 	// continue (either the trapflag was clear anyways, or the int1 cleared it)
