@@ -313,7 +313,7 @@ static void dyn_fpu_esc1(){
 			break;
 		case 0x04: /* FLDENV */
 			dyn_fill_ea(FC_ADDR);
-			gen_call_function_R(FPU_FLDENV,FC_ADDR);
+			gen_call_function_RI(FPU_FLDENV, FC_ADDR, !decode.big_op);
 			break;
 		case 0x05: /* FLDCW */
 			dyn_fill_ea(FC_ADDR);
@@ -321,7 +321,7 @@ static void dyn_fpu_esc1(){
 			break;
 		case 0x06: /* FSTENV */
 			dyn_fill_ea(FC_ADDR);
-			gen_call_function_R(FPU_FSTENV,FC_ADDR);
+			gen_call_function_RI(FPU_FSTENV, FC_ADDR, !decode.big_op);
 			break;
 		case 0x07:  /* FNSTCW*/
 			dyn_fill_ea(FC_ADDR);
@@ -533,11 +533,11 @@ static void dyn_fpu_esc5(){
 			break;
 		case 0x04:	/* FRSTOR */
 			dyn_fill_ea(FC_ADDR); 
-			gen_call_function_R(FPU_FRSTOR,FC_ADDR);
+			gen_call_function_RI(FPU_FRSTOR, FC_ADDR, !decode.big_op);
 			break;
 		case 0x06:	/* FSAVE */
 			dyn_fill_ea(FC_ADDR); 
-			gen_call_function_R(FPU_FSAVE,FC_ADDR);
+			gen_call_function_RI(FPU_FSAVE, FC_ADDR, !decode.big_op);
 			break;
 		case 0x07:   /*FNSTSW */
 			dyn_fill_ea(FC_OP1); 
@@ -566,7 +566,7 @@ static void dyn_fpu_esc6(){
 		case 0x02:  /* FCOMP5*/
 			dyn_fpu_top();
 			gen_call_function_RR(FPU_FCOM,FC_OP1,FC_OP2);
-			break;	/* TODO IS THIS ALLRIGHT ????????? */
+			break;	/* TODO IS THIS ALRIGHT ????????? */
 		case 0x03:  /*FCOMPP*/
 			if(decode.modrm.rm != 1) {
 				LOG(LOG_FPU,LOG_WARN)("ESC 6:Unhandled group %d subfunction %d",(unsigned int)decode.modrm.reg,(unsigned int)decode.modrm.rm);
