@@ -172,10 +172,10 @@ static OpCode OpCodeTable[1024]={
 {D_SETALC	,0			,0		,0		},{D_XLAT	,0			,0		,0		},
 //TODO FPU
 /* 0xd8 - 0xdf */
-{L_MODRM	,O_FPU		,0		,0		},{L_MODRM	,O_FPU		,1		,0		},
-{L_MODRM	,O_FPU		,2		,0		},{L_MODRM	,O_FPU		,3		,0		},
-{L_MODRM	,O_FPU		,4		,0		},{L_MODRM	,O_FPU		,5		,0		},
-{L_MODRM	,O_FPU		,6		,0		},{L_MODRM	,O_FPU		,7		,0		},
+{L_MODRM	,O_FPU		,0		,true   },{L_MODRM	,O_FPU		,1		,true   },
+{L_MODRM	,O_FPU		,2		,true   },{L_MODRM	,O_FPU		,3		,true   },
+{L_MODRM	,O_FPU		,4		,true   },{L_MODRM	,O_FPU		,5		,true   },
+{L_MODRM	,O_FPU		,6		,true   },{L_MODRM	,O_FPU		,7		,true   },
 
 /* 0xe0 - 0xe7 */
 {L_Ibx		,O_LOOPNZ	,S_AIPw	,0		},{L_Ibx	,O_LOOPZ	,S_AIPw	,0		},
@@ -527,10 +527,10 @@ static OpCode OpCodeTable[1024]={
 {L_Ib		,O_AAM		,0		,0		},{L_Ib		,O_AAD		,0		,0		},
 {D_SETALC	,0			,0		,0		},{D_XLAT	,0			,0		,0		},
 /* 0x2d8 - 0x2df */
-{L_MODRM	,O_FPU		,0		,0		},{L_MODRM	,O_FPU		,1		,0		},
-{L_MODRM	,O_FPU		,2		,0		},{L_MODRM	,O_FPU		,3		,0		},
-{L_MODRM	,O_FPU		,4		,0		},{L_MODRM	,O_FPU		,5		,0		},
-{L_MODRM	,O_FPU		,6		,0		},{L_MODRM	,O_FPU		,7		,0		},
+{L_MODRM	,O_FPU		,0		,false  },{L_MODRM	,O_FPU		,1		,false  },
+{L_MODRM	,O_FPU		,2		,false  },{L_MODRM	,O_FPU		,3		,false  },
+{L_MODRM	,O_FPU		,4		,false  },{L_MODRM	,O_FPU		,5		,false  },
+{L_MODRM	,O_FPU		,6		,false  },{L_MODRM	,O_FPU		,7		,false  },
 
 /* 0x2e0 - 0x2e7 */
 {L_Ibx		,O_LOOPNZ	,S_AIPd	,0		},{L_Ibx	,O_LOOPZ	,S_AIPd	,0		},
