@@ -438,7 +438,11 @@ private:
 						break;
 					case 10:															// Linefeed (combination)
 					case 13:
+#if defined(WIN32)
 						fwrite("\x0d\x00\x0a\x00", 1, 4, fh);
+#else
+						fwrite("x0a\x00", 1, 2, fh);
+#endif
 						if (i < rawdata.size() -1 && textChar == 23-rawdata[i+1])
 							i++;
 						break;
@@ -481,6 +485,7 @@ private:
             std::string result="";
             std::istringstream iss(contents.c_str());
             for (std::string token; std::getline(iss, token); ) {
+                if (token.size() && token.back() == 13) token.pop_back();
                 char* uname = CodePageGuestToHost(token.c_str());
                 result+=(uname!=NULL?std::string(uname):token)+std::string(1, 10);
             }
@@ -553,7 +558,7 @@ public:
 			*(datadst++) = ' ';
 		if (uint16_t newsize = (uint16_t)(datadst - data))									// If data
 			{
-			if (rawdata.capacity() < 100000)											// Prevent repetive size allocations
+			if (rawdata.capacity() < 100000)											// Prevent repetitive size allocations
 				rawdata.reserve(100000);
 			rawdata.append((char *)data, newsize);
 			}
@@ -700,7 +705,7 @@ uint8_t DOS_FindDevice(char const * name) {
 	if(name_part) {
 		*name_part++ = 0;
 		//Check validity of leading directory.
-//		if(!Drives[drive]->TestDir(fullname)) return DOS_DEVICES; //can be invalid
+		if(!Drives[drive]->TestDir(fullname) && !strcasecmp(name_part, "NUL")) return DOS_DEVICES; //can be invalid
 	} else name_part = fullname;
    
 	char* dot = strrchr(name_part,'.');
