@@ -379,7 +379,7 @@ bool DOS_Execute(const char* name, PhysPt block_pt, uint8_t flags) {
 				pos=0;DOS_SeekFile(fhandle,&pos,DOS_SEEK_SET);	
 				uint16_t dataread=0x1800;
 				DOS_ReadFile(fhandle,loadbuf,&dataread);
-				if (dataread<0x1800) maxsize=dataread;
+				if (dataread<0x1800) maxsize=((dataread+0x10)>>4)+0x20;
 				if (minsize>maxsize) minsize=maxsize;
 			}
 		} else {	/* Exe size calculated from header */
@@ -408,6 +408,7 @@ bool DOS_Execute(const char* name, PhysPt block_pt, uint8_t flags) {
 			maxsize=0xffff;
 			/* resize to full extent of memory block */
 			DOS_ResizeMemory(pspseg,&maxsize);
+			memsize=maxsize;
 		}
 		loadseg=pspseg+16;
 		if (!iscom) {
@@ -470,7 +471,7 @@ bool DOS_Execute(const char* name, PhysPt block_pt, uint8_t flags) {
 		SetupPSP(pspseg,memsize,envseg);
 		SetupCMDLine(pspseg,block);
 	}
-	CALLBACK_SCF(false);		/* Carry flag cleared for caller if successfull */
+	CALLBACK_SCF(false);		/* Carry flag cleared for caller if successful */
     if(flags == OVERLAY) {
         reg_ax = 0;             // Testing with MS-DOS (6.22) shows that if INT 21 AX=4B is called with the overlay flag (AL==3), then AX and DX are 0 on return.
         reg_dx = 0;
