@@ -96,7 +96,7 @@ void DOS_SetDefaultDrive(uint8_t drive) {
 
 bool DOS_MakeName(char const * const name,char * const fullname,uint8_t * drive) {
 	if(!name || *name == 0 || *name == ' ' || *name == '\n' || *name == ':') {
-		/* Both \0 and space are seperators and
+		/* Both \0 and space are separators and
 		 * empty filenames report file not found */
 		DOS_SetError(DOSERR_FILE_NOT_FOUND);
 		return false;
@@ -247,7 +247,7 @@ bool DOS_MakeName(char const * const name,char * const fullname,uint8_t * drive)
 				if (ext) {
 					if(strchr(ext+1,'.')) { 
 					//another dot in the extension =>file not found
-					//Or path not found depending on wether 
+					//Or path not found depending on whether
 					//we are still in dir check stage or file stage
 						if(stop)
 							DOS_SetError(DOSERR_FILE_NOT_FOUND);
@@ -282,7 +282,7 @@ bool DOS_GetSFNPath(char const * const path,char * SFNPath,bool LFN) {
     char pdir[LFN_NAMELENGTH+4], *p;
     uint8_t drive;char fulldir[DOS_PATHLENGTH],LFNPath[CROSS_LEN];
     char name[DOS_NAMELENGTH_ASCII], lname[LFN_NAMELENGTH];
-    uint32_t size;uint16_t date;uint16_t time;uint8_t attr;
+    uint32_t size,hsize;uint16_t date;uint16_t time;uint8_t attr;
     if (!DOS_MakeName(path,fulldir,&drive)) return false;
 #if defined(WIN32) && !(defined(__MINGW32__) && !defined(__MINGW64_VERSION_MAJOR))
 	if (Network_IsNetworkResource(fulldir)) {
@@ -310,7 +310,7 @@ bool DOS_GetSFNPath(char const * const path,char * SFNPath,bool LFN) {
 			lfn_filefind_handle=LFN_FILEFIND_INTERNAL;
 			if (DOS_FindFirst(pdir,0xffff & DOS_ATTR_DIRECTORY & ~DOS_ATTR_VOLUME,false)) {
 				lfn_filefind_handle=fbak;
-				dta.GetResult(name,lname,size,date,time,attr);
+				dta.GetResult(name,lname,size,hsize,date,time,attr);
 				strcat(SFNPath,name);
 				strcat(LFNPath,lname);
 				strcat(SFNPath,"\\");
@@ -335,7 +335,7 @@ bool DOS_GetSFNPath(char const * const path,char * SFNPath,bool LFN) {
 		sprintf(pdir,"\"%s%s\"",SFNPath,p);
 		lfn_filefind_handle=LFN_FILEFIND_INTERNAL;
 		if (!strrchr(p,'*')&&!strrchr(p,'?')&&DOS_FindFirst(pdir,0xffff & ~DOS_ATTR_VOLUME,false)) {
-			dta.GetResult(name,lname,size,date,time,attr);
+			dta.GetResult(name,lname,size,hsize,date,time,attr);
 			strcat(SFNPath,name);
 			strcat(LFNPath,lname);
         } else if (checkwat) {
@@ -551,7 +551,7 @@ bool DOS_FindFirst(const char * search,uint16_t attr,bool fcb_findfirst) {
 	char dir[DOS_PATHLENGTH];char pattern[DOS_PATHLENGTH];
 	size_t len = strlen(search);
 	if(len && search[len - 1] == '\\' && !( (len > 2) && (search[len - 2] == ':') && (attr == DOS_ATTR_VOLUME) )) { 
-		//Dark Forces installer, but c:\ is allright for volume labels(exclusively set)
+		//Dark Forces installer, but c:\ is alright for volume labels(exclusively set)
 		DOS_SetError(DOSERR_NO_MORE_FILES);
 		return false;
 	}
@@ -598,7 +598,7 @@ bool DOS_FindFirst(const char * search,uint16_t attr,bool fcb_findfirst) {
 		find_last = strrchr(pattern,'.');
 		if(find_last) *find_last = 0;
 		//TODO use current date and time
-        dta.SetResult(pattern,pattern,0,0,0,DOS_ATTR_DEVICE);
+        dta.SetResult(pattern,pattern,0,0,0,0,DOS_ATTR_DEVICE);
 		LOG(LOG_DOSMISC,LOG_WARN)("finding device %s",pattern);
 		return true;
 	}
@@ -1006,8 +1006,8 @@ bool DOS_UnlinkFile(char const * const name) {
 		bool ret=DOS_FindFirst(((pfull.length()&&pfull[0]=='"'?"":"\"")+pfull+(pfull.length()&&pfull[pfull.length()-1]=='"'?"":"\"")).c_str(),0xffu & ~DOS_ATTR_VOLUME & ~DOS_ATTR_DIRECTORY);
 		if (ret) do {
 			char find_name[DOS_NAMELENGTH_ASCII],lfind_name[LFN_NAMELENGTH];
-			uint16_t find_date,find_time;uint32_t find_size;uint8_t find_attr;
-			dta.GetResult(find_name,lfind_name,find_size,find_date,find_time,find_attr);
+			uint16_t find_date,find_time;uint32_t find_size,find_hsize;uint8_t find_attr;
+			dta.GetResult(find_name,lfind_name,find_size,find_hsize,find_date,find_time,find_attr);
 			if (!(find_attr & DOS_ATTR_DIRECTORY)&&strlen(find_name)&&!strchr(find_name, '*')&&!strchr(find_name, '?')) {
 				strcpy(temp, dir);
 				if (strlen(temp)&&temp[strlen(temp)-1]!='\\') strcat(temp, "\\");
@@ -1534,13 +1534,13 @@ void DTAExtendName(char * const name,char * const filename,char * const ext) {
 static void SaveFindResult(DOS_FCB & find_fcb) {
 	DOS_DTA find_dta(dos.tables.tempdta);
     char name[DOS_NAMELENGTH_ASCII],lname[LFN_NAMELENGTH];
-    uint32_t size;uint16_t date;uint16_t time;uint8_t attr;uint8_t drive;
+    uint32_t size,hsize;uint16_t date;uint16_t time;uint8_t attr;uint8_t drive;
 	char file_name[9];char ext[4];
-	find_dta.GetResult(name,lname,size,date,time,attr);
+	find_dta.GetResult(name,lname,size,hsize,date,time,attr);
 	drive=find_fcb.GetDrive()+1;
 	uint8_t find_attr = DOS_ATTR_ARCHIVE;
 	find_fcb.GetAttr(find_attr); /* Gets search attributes if extended */
-	/* Create a correct file and extention */
+	/* Create a correct file and extension */
     if (attr & DOS_ATTR_VOLUME)
         DTAExtendNameVolumeLabel(name,file_name,ext);
     else
@@ -1583,8 +1583,8 @@ bool DOS_FCBOpen(uint16_t seg,uint16_t offset) {
 		DOS_DTA find_dta(dos.tables.tempdta);
 		DOS_FCB find_fcb(RealSeg(dos.tables.tempdta),RealOff(dos.tables.tempdta));
 		char name[DOS_NAMELENGTH_ASCII],lname[LFN_NAMELENGTH],file_name[9],ext[4];
-		uint32_t size;uint16_t date,time;uint8_t attr;
-		find_dta.GetResult(name,lname,size,date,time,attr);
+		uint32_t size,hsize;uint16_t date,time;uint8_t attr;
+		find_dta.GetResult(name,lname,size,hsize,date,time,attr);
 		DTAExtendName(name,file_name,ext);
 		find_fcb.SetName(fcb.GetDrive()+1,file_name,ext);
 		find_fcb.GetName(shortname);
@@ -1735,7 +1735,7 @@ uint8_t DOS_FCBIncreaseSize(uint16_t seg,uint16_t offset) {
 }
 
 uint8_t DOS_FCBRandomRead(uint16_t seg,uint16_t offset,uint16_t * numRec,bool restore) {
-/* if restore is true :random read else random blok read. 
+/* if restore is true :random read else random block read.
  * random read updates old block and old record to reflect the random data
  * before the read!!!!!!!!! and the random data is not updated! (user must do this)
  * Random block read updates these fields to reflect the state after the read!
@@ -2422,7 +2422,7 @@ void POD_Load_DOS_Files( std::istream& stream )
                     MSCDEX_SetCDInterface(CDROM_USE_SDL, -1);
                     uint8_t mediaid = 0xF8;
                     int error = -1;
-                    isoDrive* newDrive = new isoDrive('A'+lcv, dinfo+9, mediaid, error);
+                    isoDrive* newDrive = new isoDrive('A'+lcv, dinfo+9, mediaid, error, options);
                     if (!error) {
                         Drives[lcv] = newDrive;
                         DriveManager::AppendDisk(lcv, newDrive);
@@ -2556,10 +2556,11 @@ void POD_Load_DOS_Files( std::istream& stream )
             if( Drives[lcv] && strcasecmp(Drives[lcv]->info, dinfo))
                 unmount(lcv);
             if (!Drives[lcv] && !strncmp(dinfo,"isoDrive ",9)) {
+                std::vector<std::string> options;
                 MSCDEX_SetCDInterface(CDROM_USE_SDL, -1);
                 uint8_t mediaid = 0xF8;
                 int error = -1;
-                isoDrive* newDrive = new isoDrive('A'+lcv, dinfo+9, mediaid, error);
+                isoDrive* newDrive = new isoDrive('A'+lcv, dinfo+9, mediaid, error, options);
                 if (!error) {
                     Drives[lcv] = newDrive;
                     DriveManager::AppendDisk(lcv, newDrive);
