@@ -39,6 +39,10 @@ bool ANSI_SYS_installed();
 #if defined(MACOSX)
 bool SetClipboard(std::string value);
 #endif
+#if (!defined(WIN32) && defined(C_SDL2)) || defined(MACOSX)
+typedef char host_cnv_char_t;
+host_cnv_char_t *CodePageGuestToHost(const char *s);
+#endif
 
 extern bool enable_share_exe, enable_network_redirector;
 
@@ -267,7 +271,7 @@ static bool DOS_MultiplexFunctions(void) {
     case 0x1600:    /* Windows enhanced mode installation check */
         // Leave AX as 0x1600, indicating that neither Windows 3.x enhanced mode, Windows/386 2.x
         // nor Windows 95 are running, nor is XMS version 1 driver installed
-		if (!control->SecureMode() && ((reg_sp == 0xFFF6 && mem_readw(SegPhys(ss)+reg_sp) == 0x142A) || !strcmp(RunningProgram, "DOSCLIP"))) // Hack for DOSCLIP
+		if (!control->SecureMode() && ((reg_sp == 0xFFF6 && mem_readw(SegPhys(ss)+reg_sp) == 0x142A) || (reg_sp == 0xFF88 && mem_readw(SegPhys(ss)+reg_sp) == 0xFF9D) || !strcmp(RunningProgram, "DOSCLIP") || !strcmp(RunningProgram, "TOCLIP"))) // Hack for DOSCLIP/TOCLIP
 			reg_ax = 0x301;
         return true;
 	case 0x1605:	/* Windows init broadcast */
@@ -354,7 +358,7 @@ static bool DOS_MultiplexFunctions(void) {
 					reg_bx = (reg_dx & 0x16);
 					reg_dx = 0xa2ab;
 					return true;
-				case 0x0003:		// get size of data struc
+				case 0x0003:		// get size of data struct
 					if (reg_dx==0x0001) {
 						// CDS size requested
 						reg_ax = 0xb97c;
@@ -469,8 +473,7 @@ static bool DOS_MultiplexFunctions(void) {
             std::istringstream iss(text);
             std::string result="";
             for (std::string token; std::getline(iss, token); ) {
-                typedef char host_cnv_char_t;
-                host_cnv_char_t *CodePageGuestToHost(const char *s);
+                if (token.size() && token.back() == 13) token.pop_back();
                 char* uname = CodePageGuestToHost(token.c_str());
                 result+=(uname!=NULL?std::string(uname):token)+std::string(1, 10);
             }
