@@ -155,7 +155,7 @@ void DOS_Drive_Cache::SetBaseDir(const char* baseDir, DOS_Drive *drive) {
     UINT test = GetDriveType(drives);
     if(test == DRIVE_CDROM) cdrom = true;
 #else // OS2
-    //TODO determine wether cdrom or not!
+    //TODO determine whether cdrom or not!
     FSINFO fsinfo;
     ULONG drivenumber = drive[0];
     if (drivenumber > 26) { // drive letter was lowercase
@@ -622,7 +622,7 @@ void DOS_Drive_Cache::CreateShortName(CFileInfo* curDir, CFileInfo* info) {
         unsigned int i = 0;
         bool lead = false;
         while (tmpName[i] != 0) {
-                if ((tmpName[i]&0xFF)<=32||tmpName[i]==127||tmpName[i]=='"'||tmpName[i]=='+'||tmpName[i]=='='||tmpName[i]==','||tmpName[i]==';'||tmpName[i]==':'||tmpName[i]=='<'||tmpName[i]=='>'||((tmpName[i]=='['||tmpName[i]==']'||tmpName[i]=='|')&&(!lead||((dos.loaded_codepage==936||IS_PDOSV)&&!gbk)))||tmpName[i]=='?'||tmpName[i]=='*') tmpName[i]='_';
+                if ((tmpName[i]&0xFF)<=32||tmpName[i]==127||tmpName[i]=='"'||tmpName[i]=='+'||tmpName[i]=='='||tmpName[i]==','||tmpName[i]==';'||tmpName[i]==':'||tmpName[i]=='<'||tmpName[i]=='>'||((tmpName[i]=='['||tmpName[i]==']'||tmpName[i]=='|'||tmpName[i]=='\\')&&(!lead||((dos.loaded_codepage==936||IS_PDOSV)&&!gbk)))||tmpName[i]=='?'||tmpName[i]=='*') tmpName[i]='_';
                 if (lead) lead = false;
                 else if ((IS_PC98_ARCH && shiftjis_lead_byte(tmpName[i]&0xFF)) || (isDBCSCP() && isKanji1_gbk(tmpName[i]&0xFF))) lead = true;
                 i++;
@@ -776,15 +776,17 @@ DOS_Drive_Cache::CFileInfo* DOS_Drive_Cache::FindDirInfo(const char* path, char*
     } while (pos);
 
     // Save last result for faster access next time
-    strcpy(save_path,path);
-    strcpy(save_expanded,expandedPath);
-    save_dir = curDir;
+    if (strlen(expandedPath) < CROSS_LEN) {
+        strcpy(save_path,path);
+        strcpy(save_expanded,expandedPath);
+        save_dir = curDir;
+    }
 
     return curDir;
 }
 
 bool DOS_Drive_Cache::OpenDir(const char* path, uint16_t& id) {
-    char expand[CROSS_LEN] = {0};
+    char expand[CROSS_LEN + LFN_NAMELENGTH] = {0};
     CFileInfo* dir = FindDirInfo(path,expand);
     if (OpenDir(dir,expand,id)) {
         dirSearch[id]->nextEntry = 0;
@@ -794,6 +796,7 @@ bool DOS_Drive_Cache::OpenDir(const char* path, uint16_t& id) {
 }
 
 bool DOS_Drive_Cache::OpenDir(CFileInfo* dir, const char* expand, uint16_t& id) {
+    if (strlen(expand)>=CROSS_LEN-1) return false;
     id = GetFreeID(dir);
     dirSearch[id] = dir;
     char expandcopy [CROSS_LEN];
@@ -872,7 +875,7 @@ void DOS_Drive_Cache::CopyEntry(CFileInfo* dir, CFileInfo* from) {
 }
 
 bool DOS_Drive_Cache::ReadDir(uint16_t id, char* &result, char * &lresult) {
-    // shouldnt happen...
+    // shouldn't happen...
     if (id>=MAX_OPENDIRS) return false;
 
     if (!IsCachedIn(dirSearch[id])) {
