@@ -30,6 +30,8 @@
 #include "setup.h"
 #include "control.h"
 #include "mapper.h"
+#include "menudef.h"
+#include "pic.h"
 #include "cross.h"
 #include "hardware.h"
 #include "support.h"
@@ -59,8 +61,9 @@ bool                                    systemmessagebox(char const * aTitle, ch
 
 #if defined(USE_TTF)
 bool resetreq=false;
-void resetFontSize();
+void resetFontSize(), drawmenu(Bitu val);
 #endif
+void SetWindowTransparency(int trans);
 
 static void Check_Palette(void) {
     /* Clean up any previous changed palette data */
@@ -820,14 +823,24 @@ forcenormal:
     render.scale.outWrite = 0;
     /* Signal the next frame to first reinit the cache */
     render.scale.clearCache = true;
-    render.active=true;
+
+    if (!sdl.window_too_small)
+        render.active=true;
 
     last_gfx_flags = gfx_flags;
 #if defined(USE_TTF)
-    if (sdl.desktop.want_type == SCREEN_TTF && resetreq) resetFontSize();
+    if (sdl.desktop.want_type == SCREEN_TTF) {
+        if (resetreq) resetFontSize();
+#if defined(HX_DOS)
+        else PIC_AddEvent(drawmenu, 200);
+#endif
+    }
 #endif
     // Ensure VMware mouse support knows the current parameters
 	VMWARE_ScreenParams(sdl.clip.x, sdl.clip.y, sdl.clip.w, sdl.clip.h, sdl.desktop.fullscreen);
+#if defined(MACOSX) && !defined(C_SDL2)
+	SetWindowTransparency(-1);
+#endif
 }
 
 void RENDER_CallBack( GFX_CallBackFunctions_t function ) {
@@ -1159,7 +1172,7 @@ std::string LoadGLShader(Section_prop * section) {
         render.shader_def = f=="default";
         if (initgl==2) sdl_opengl.use_shader=true;
     } else if (!RENDER_GetShader(sh->realpath,(char *)shader_src.c_str())) {
-        std::string path = std::string("glshaders") + CROSS_FILESPLIT + f;
+        std::string path = std::string("glshaders") + CROSS_FILESPLIT + f, pathcfg, pathres;
         if (!RENDER_GetShader(path,(char *)shader_src.c_str())) {
             std::string exePath = GetDOSBoxXPath();
             if (exePath.size()) path = exePath + std::string("glshaders") + CROSS_FILESPLIT + f;
@@ -1171,8 +1184,10 @@ std::string LoadGLShader(Section_prop * section) {
         }
         if (path.size() && !RENDER_GetShader(path,(char *)shader_src.c_str())) {
             Cross::GetPlatformConfigDir(path);
-            path = path + "glshaders" + CROSS_FILESPLIT + f;
-            if (!RENDER_GetShader(path,(char *)shader_src.c_str()) && (sh->realpath==f || !RENDER_GetShader(f,(char *)shader_src.c_str()))) {
+            pathcfg = path + "glshaders" + CROSS_FILESPLIT + f;
+            Cross::GetPlatformResDir(path);
+            pathres = path + "glshaders" + CROSS_FILESPLIT + f;
+            if (!RENDER_GetShader(pathcfg,(char *)shader_src.c_str()) && !RENDER_GetShader(pathres,(char *)shader_src.c_str()) && (sh->realpath==f || !RENDER_GetShader(f,(char *)shader_src.c_str()))) {
                 sh->SetValue("none");
                 LOG_MSG("Shader file \"%s\" not found", f.c_str());
             } else {
