@@ -335,124 +335,129 @@ void BIOS_Post_register_parports() {
 }
 	
 class PARPORTS:public Module_base {
-public:
-	
-	PARPORTS (Section * configuration):Module_base (configuration) {
+	public:
+		PARPORTS (Section * configuration):Module_base (configuration) {
 
-        // TODO: PC-98 does have one parallel port, if at all
-        if (IS_PC98_ARCH) return;
+			// TODO: PC-98 does have one parallel port, if at all
+			if (IS_PC98_ARCH) return;
 
 #if C_PRINTER
-        // we can only have one printer redirection, hence the variable
-        printer_used = false;
+			// we can only have one printer redirection, hence the variable
+			printer_used = false;
 #endif
-        disneyport = 0;
+			disneyport = 0;
 
-		// default ports & interrupts
-		uint8_t defaultirq[] = { 7, 5, 12, 0, 0, 0, 0, 0, 0};
-		Section_prop *section = static_cast <Section_prop*>(configuration);
-		
-		char pname[]="parallelx";
-		// iterate through all 3 lpt ports
-		for (Bitu i = 0; i < 9; i++) {
-			pname[8] = '1' + (char)i;
-			CommandLine cmd(0,section->Get_string(pname));
-            CommandLine tmp(0,section->Get_string(pname), CommandLine::either, true);
+			// default ports & interrupts
+			uint8_t defaultirq[] = { 7, 5, 12, 0, 0, 0, 0, 0, 0};
+			Section_prop *section = static_cast <Section_prop*>(configuration);
 
-			std::string str;
-            bool squote = false;
-            // single quotes to quote string?
-            if(cmd.FindStringBegin("squote",str,false)) {
-                squote = true;
-                cmd=tmp;
-            }
+			char pname[]="parallelx";
+			// iterate through all 3 lpt ports
+			for (Bitu i = 0; i < 9; i++) {
+				pname[8] = '1' + (char)i;
+				CommandLine cmd(0,section->Get_string(pname));
+				CommandLine tmp(0,section->Get_string(pname), CommandLine::either, true);
+
+				std::string str;
+				bool squote = false;
+				// single quotes to quote string?
+				if(cmd.FindStringBegin("squote",str,false)) {
+					squote = true;
+					cmd=tmp;
+				}
+
+				if(cmd.FindStringBegin("base:",str,true))
+					parallel_baseaddr[i] = (uint16_t)strtol(str.c_str(), NULL, 16);
+				if(cmd.FindStringBegin("irq:",str,true))
+					defaultirq[i] = (uint8_t)strtol(str.c_str(), NULL, 10);
+				cmd.FindCommand(1,str);
+
+				/* DOSBox SVN disney=true compatability: Reserve LPT1 for Disney Sound Source */
+				if (i == 0 && DISNEY_ShouldInit())
+					continue;
 
-            if(cmd.FindStringBegin("base:",str,true))
-				parallel_baseaddr[i] = (uint16_t)strtol(str.c_str(), NULL, 16);
-            if(cmd.FindStringBegin("irq:",str,true))
-				defaultirq[i] = (uint8_t)strtol(str.c_str(), NULL, 10);
-			cmd.FindCommand(1,str);
 #if C_DIRECTLPT
-			if(str=="reallpt") {
-				CDirectLPT* cdlpt= new CDirectLPT(i, defaultirq[i],&cmd);
-				if(cdlpt->InstallationSuccessful) {
-					parallelPortObjects[i]=cdlpt;
-					parallelPortObjects[i]->parallelType = PARALLEL_TYPE_REALLPT;
-					cmd.Shift(1);
-					cmd.GetStringRemain(parallelPortObjects[i]->commandLineString);
-				} else {
-					delete cdlpt;
-					parallelPortObjects[i]=0;
-				}
-			} else
+				if(str=="reallpt") {
+					CDirectLPT* cdlpt= new CDirectLPT(i, defaultirq[i],&cmd);
+					if(cdlpt->InstallationSuccessful) {
+						parallelPortObjects[i]=cdlpt;
+						parallelPortObjects[i]->parallelType = PARALLEL_TYPE_REALLPT;
+						cmd.Shift(1);
+						cmd.GetStringRemain(parallelPortObjects[i]->commandLineString);
+					} else {
+						delete cdlpt;
+						parallelPortObjects[i]=0;
+					}
+				} else
 #endif
-			if(!str.compare("file")) {
-				CFileLPT* cflpt= new CFileLPT(i, defaultirq[i], &cmd, squote);
-				if(cflpt->InstallationSuccessful) {
-					parallelPortObjects[i]=cflpt;
-					parallelPortObjects[i]->parallelType = PARALLEL_TYPE_FILE;
-					cmd.Shift(1);
-					cmd.GetStringRemain(parallelPortObjects[i]->commandLineString);
-				} else {
-					delete cflpt;
-					parallelPortObjects[i]=0;
-				}
-			} else
+					if(!str.compare("file")) {
+						CFileLPT* cflpt= new CFileLPT(i, defaultirq[i], &cmd, squote);
+						if(cflpt->InstallationSuccessful) {
+							parallelPortObjects[i]=cflpt;
+							parallelPortObjects[i]->parallelType = PARALLEL_TYPE_FILE;
+							cmd.Shift(1);
+							cmd.GetStringRemain(parallelPortObjects[i]->commandLineString);
+						} else {
+							delete cflpt;
+							parallelPortObjects[i]=0;
+						}
+					} else
 #if C_PRINTER
-            // allow printer redirection on a single port
-            if (str == "printer" && !printer_used)
-            {
-                CPrinterRedir* cprd = new CPrinterRedir(i, defaultirq[i], &cmd);
-                if (cprd->InstallationSuccessful)
-                {
-                    parallelPortObjects[i] = cprd;
-					parallelPortObjects[i]->parallelType = PARALLEL_TYPE_PRINTER;
-					cmd.Shift(1);
-					cmd.GetStringRemain(parallelPortObjects[i]->commandLineString);
-                    printer_used = true;
-                }
-                else
-                {
-                    LOG_MSG("Error: printer is not enabled.");
-                    delete cprd;
-                    parallelPortObjects[i] = 0;
-                }
-            } else
+						// allow printer redirection on a single port
+						if (str == "printer" && !printer_used)
+						{
+							CPrinterRedir* cprd = new CPrinterRedir(i, defaultirq[i], &cmd);
+							if (cprd->InstallationSuccessful)
+							{
+								parallelPortObjects[i] = cprd;
+								parallelPortObjects[i]->parallelType = PARALLEL_TYPE_PRINTER;
+								cmd.Shift(1);
+								cmd.GetStringRemain(parallelPortObjects[i]->commandLineString);
+								printer_used = true;
+							}
+							else
+							{
+								LOG_MSG("Error: printer is not enabled.");
+								delete cprd;
+								parallelPortObjects[i] = 0;
+							}
+						} else
 #endif				
-            if(str=="disabled") {
-				parallelPortObjects[i] = 0;
-			} else if (str == "disney") {
-				if (!DISNEY_HasInit()) {
-					LOG_MSG("LPT%d: User explicitly assigned Disney Sound Source to this port",(int)i+1);
-					DISNEY_Init(parallel_baseaddr[i]);
-                    if (DISNEY_HasInit()) disneyport = i+1;
-				}
-				else {
-					LOG_MSG("LPT%d: Disney Sound Source already initialized on a port, cannot init again",(int)i+1);
-				}
-			} else {
-				LOG_MSG ("Invalid type for LPT%d.",(int)i + 1);
-				parallelPortObjects[i] = 0;
-			}
-		} // for lpt 1-9
-	}
-#if C_PRINTER
-	// we can only have one printer redirection, hence the variable
-	bool printer_used = false;
-#endif
-
-	~PARPORTS () {
-//		LOG_MSG("Parports destructor\n");
-		for (Bitu i = 0; i < 9; i++) {
-			if (parallelPortObjects[i]) {
-				delete parallelPortObjects[i];
-				parallelPortObjects[i] = 0;
-			}
+							if(str=="disabled") {
+								parallelPortObjects[i] = 0;
+							} else if (str == "disney") {
+								parallelPortObjects[i] = 0;
+								if (!DISNEY_HasInit()) {
+									LOG_MSG("LPT%d: User explicitly assigned Disney Sound Source to this port",(int)i+1);
+									DISNEY_Init(parallel_baseaddr[i]);
+									if (DISNEY_HasInit()) disneyport = i+1;
+								}
+								else {
+									LOG_MSG("LPT%d: Disney Sound Source already initialized on a port, cannot init again",(int)i+1);
+								}
+							} else {
+								LOG_MSG ("Invalid type for LPT%d.",(int)i + 1);
+								parallelPortObjects[i] = 0;
+							}
+			} // for lpt 1-9
 		}
 #if C_PRINTER
-		printer_used = false;
+		// we can only have one printer redirection, hence the variable
+		bool printer_used = false;
 #endif
-	}
+
+		~PARPORTS () {
+			// LOG_MSG("Parports destructor\n");
+			for (Bitu i = 0; i < 9; i++) {
+				if (parallelPortObjects[i]) {
+					delete parallelPortObjects[i];
+					parallelPortObjects[i] = 0;
+				}
+			}
+#if C_PRINTER
+			printer_used = false;
+#endif
+		}
 };
 
 static PARPORTS *testParallelPortsBaseclass = NULL;
@@ -470,10 +475,10 @@ static const char *parallelTypes[PARALLEL_TYPE_COUNT] = {
 };
 
 class PARALLEL : public Program {
-public:
-	void Run();
-private:
-	void showPort(int port);
+	public:
+		void Run();
+	private:
+		void showPort(int port);
 };
 
 void PARALLEL::showPort(int port) {
@@ -487,8 +492,8 @@ void PARALLEL::showPort(int port) {
 
 void PARALLEL::Run()
 {
-    if (!testParallelPortsBaseclass) return;
-    if (cmd->FindExist("-?", false) || cmd->FindExist("/?", false)) {
+	if (!testParallelPortsBaseclass) return;
+	if (cmd->FindExist("-?", false) || cmd->FindExist("/?", false)) {
 		WriteOut("Views or changes the parallel port settings.\n\nPARALLEL [port] [type] [option]\n\n"
 				" port   Parallel port number (between 1 and 9).\n type   Type of the parallel port, including:\n        ");
 		for (int x=0; x<PARALLEL_TYPE_COUNT; x++) {
@@ -550,19 +555,19 @@ void PARALLEL::Run()
 			commandLineString.append(temp_line);
 			commandLineString.append(" ");
 		}
-            CommandLine cmd("PARALLEL.COM",commandLineString.c_str());
-            CommandLine tmp("PARALLEL.COM",commandLineString.c_str(), CommandLine::either, true);
-            std::string str;
-            bool squote = false;
-            // single quotes to quote string?
-            if(cmd.FindStringBegin("squote",str,false)) {
-                squote = true;
-                cmd=tmp;
-            }
-			if(cmd.FindStringBegin("base:",str,true))
-				parallel_baseaddr[port-1] = (uint16_t)strtol(str.c_str(), NULL, 16);
-			if(cmd.FindStringBegin("irq:",str,true))
-				defaultirq[port-1] = (uint8_t)strtol(str.c_str(), NULL, 10);
+		CommandLine cmd("PARALLEL.COM",commandLineString.c_str());
+		CommandLine tmp("PARALLEL.COM",commandLineString.c_str(), CommandLine::either, true);
+		std::string str;
+		bool squote = false;
+		// single quotes to quote string?
+		if(cmd.FindStringBegin("squote",str,false)) {
+			squote = true;
+			cmd=tmp;
+		}
+		if(cmd.FindStringBegin("base:",str,true))
+			parallel_baseaddr[port-1] = (uint16_t)strtol(str.c_str(), NULL, 16);
+		if(cmd.FindStringBegin("irq:",str,true))
+			defaultirq[port-1] = (uint8_t)strtol(str.c_str(), NULL, 10);
 		// Remove existing port.
 		if (parallelPortObjects[port-1]) {
 #if C_PRINTER
@@ -575,13 +580,13 @@ void PARALLEL::Run()
 			if (mode==PARALLEL_TYPE_DISNEY&&disneyport!=port&&DISNEY_HasInit()) {
 				WriteOut("Disney is already assigned to a different port.\n");
 				return;
-            }
+			}
 			DOS_PSP curpsp(dos.psp());
 			if (dos.psp()!=curpsp.GetParent()) {
-                char name[5];
-                sprintf(name, "LPT%d", port);
-                curpsp.CloseFile(name);
-            }
+				char name[5];
+				sprintf(name, "LPT%d", port);
+				curpsp.CloseFile(name);
+			}
 			delete parallelPortObjects[port-1];
 			parallelPortObjects[port-1] = 0;
 #if C_PRINTER
@@ -593,14 +598,14 @@ void PARALLEL::Run()
 			WriteOut("Disney is already assigned to a different port.\n");
 			return;
 		} else if (disneyport==port) {
-            if (mode==PARALLEL_TYPE_DISNEY) {
-                showPort(port-1);
-                return;
-            } else {
-                DISNEY_Close();
-                if (!DISNEY_HasInit()) disneyport=0;
-            }
-        }
+			if (mode==PARALLEL_TYPE_DISNEY) {
+				showPort(port-1);
+				return;
+			} else {
+				DISNEY_Close();
+				if (!DISNEY_HasInit()) disneyport=0;
+			}
+		}
 		// Recreate the port with the new mode.
 		switch (mode) {
 			case PARALLEL_TYPE_DISABLED:
@@ -608,38 +613,38 @@ void PARALLEL::Run()
 				break;
 #if C_DIRECTLPT
 			case PARALLEL_TYPE_REALLPT:
-			{
-				CDirectLPT* cdlpt= new CDirectLPT(port-1, defaultirq[port-1],&cmd);
-				if(cdlpt->InstallationSuccessful) parallelPortObjects[port-1]=cdlpt;
-				break;
-			}
+				{
+					CDirectLPT* cdlpt= new CDirectLPT(port-1, defaultirq[port-1],&cmd);
+					if(cdlpt->InstallationSuccessful) parallelPortObjects[port-1]=cdlpt;
+					break;
+				}
 #endif
 			case PARALLEL_TYPE_FILE:
-			{
-				CFileLPT* cflpt= new CFileLPT(port-1, defaultirq[port-1], &cmd, squote);
-				if(cflpt->InstallationSuccessful) parallelPortObjects[port-1]=cflpt;
-				break;
-			}
+				{
+					CFileLPT* cflpt= new CFileLPT(port-1, defaultirq[port-1], &cmd, squote);
+					if(cflpt->InstallationSuccessful) parallelPortObjects[port-1]=cflpt;
+					break;
+				}
 #if C_PRINTER
 			case PARALLEL_TYPE_PRINTER:
-                if (!testParallelPortsBaseclass->printer_used) {
-                    CPrinterRedir* cprd = new CPrinterRedir(port-1, defaultirq[port-1], &cmd);
-                    if(cprd->InstallationSuccessful) {
-                        parallelPortObjects[port-1]=cprd;
-                        testParallelPortsBaseclass->printer_used=true;
-                    }
-                }
-                break;
+				if (!testParallelPortsBaseclass->printer_used) {
+					CPrinterRedir* cprd = new CPrinterRedir(port-1, defaultirq[port-1], &cmd);
+					if(cprd->InstallationSuccessful) {
+						parallelPortObjects[port-1]=cprd;
+						testParallelPortsBaseclass->printer_used=true;
+					}
+				}
+				break;
 #endif
 			case PARALLEL_TYPE_DISNEY:
 				if (!DISNEY_HasInit()) {
-                    DISNEY_Init(parallel_baseaddr[port-1]);
-                    if (DISNEY_HasInit()) disneyport=port;
-                }
+					DISNEY_Init(parallel_baseaddr[port-1]);
+					if (DISNEY_HasInit()) disneyport=port;
+				}
 				break;
 		}
 		if (parallelPortObjects[port-1]) {
-            parallelPortObjects[port-1]->registerDOSDevice();
+			parallelPortObjects[port-1]->registerDOSDevice();
 			parallelPortObjects[port-1]->parallelType = (ParallelTypesE)mode;
 			parallelPortObjects[port-1]->commandLineString = commandLineString;
 		}
@@ -662,7 +667,7 @@ void runParallel(const char *str) {
 }
 
 void PARALLEL_Destroy (Section * sec) {
-    (void)sec;//UNUSED
+	(void)sec;//UNUSED
 	if (testParallelPortsBaseclass != NULL) {
 		delete testParallelPortsBaseclass;
 		testParallelPortsBaseclass = NULL;
@@ -670,22 +675,22 @@ void PARALLEL_Destroy (Section * sec) {
 }
 
 void PARALLEL_OnPowerOn (Section * sec) {
-    (void)sec;//UNUSED
+	(void)sec;//UNUSED
 	LOG(LOG_MISC,LOG_DEBUG)("Reinitializing parallel port emulation");
 
 	if (testParallelPortsBaseclass) delete testParallelPortsBaseclass;
 	testParallelPortsBaseclass = new PARPORTS (control->GetSection("parallel"));
 
-	/* Mainline DOSBox 0.74 compatible support for "disney=true" setting.
-	 * But, don't allocate the Disney Sound Source if LPT1 is already taken. */
+	/* DOSBox SVN compatability: LPT1 should be reserved for Disney Sound Source if disney=true, now init it */
 	if (!DISNEY_HasInit() && DISNEY_ShouldInit() && parallelPortObjects[0] == NULL) {
-		LOG_MSG("LPT: LPT1 not taken, and dosbox-x.conf says to emulate Disney Sound Source");
+		LOG_MSG("disney=true. For compatability with other DOSBox forks and SVN, LPT1 has been reserved for Disney Sound Source. Initializing it now.");
+		LOG_MSG("DOSBox-X also supports disney=false and parallel1=disney");
 		DISNEY_Init(parallel_baseaddr[0]);
 	}
 }
 
 void PARALLEL_OnDOSKernelInit (Section * sec) {
-    (void)sec;//UNUSED
+	(void)sec;//UNUSED
 	unsigned int i;
 
 	LOG(LOG_MISC,LOG_DEBUG)("DOS kernel initializing, creating LPTx devices");
@@ -697,7 +702,7 @@ void PARALLEL_OnDOSKernelInit (Section * sec) {
 }
 
 void PARALLEL_OnDOSKernelExit (Section * sec) {
-    (void)sec;//UNUSED
+	(void)sec;//UNUSED
 	unsigned int i;
 
 	for (i=0;i < 9;i++) {
@@ -707,7 +712,7 @@ void PARALLEL_OnDOSKernelExit (Section * sec) {
 }
 
 void PARALLEL_OnReset (Section * sec) {
-    (void)sec;//UNUSED
+	(void)sec;//UNUSED
 	unsigned int i;
 
 	// FIXME: Unregister/destroy the DOS devices, but consider that the DOS kernel at reset is gone.
@@ -722,11 +727,11 @@ void PARALLEL_Init () {
 
 	AddExitFunction(AddExitFunctionFuncPair(PARALLEL_Destroy),true);
 
-    if (!IS_PC98_ARCH) {
-        AddVMEventFunction(VM_EVENT_RESET,AddVMEventFunctionFuncPair(PARALLEL_OnReset));
-        AddVMEventFunction(VM_EVENT_POWERON,AddVMEventFunctionFuncPair(PARALLEL_OnPowerOn));
-        AddVMEventFunction(VM_EVENT_DOS_EXIT_BEGIN,AddVMEventFunctionFuncPair(PARALLEL_OnDOSKernelExit));
-        AddVMEventFunction(VM_EVENT_DOS_INIT_KERNEL_READY,AddVMEventFunctionFuncPair(PARALLEL_OnDOSKernelInit));
-    }
+	if (!IS_PC98_ARCH) {
+		AddVMEventFunction(VM_EVENT_RESET,AddVMEventFunctionFuncPair(PARALLEL_OnReset));
+		AddVMEventFunction(VM_EVENT_POWERON,AddVMEventFunctionFuncPair(PARALLEL_OnPowerOn));
+		AddVMEventFunction(VM_EVENT_DOS_EXIT_BEGIN,AddVMEventFunctionFuncPair(PARALLEL_OnDOSKernelExit));
+		AddVMEventFunction(VM_EVENT_DOS_INIT_KERNEL_READY,AddVMEventFunctionFuncPair(PARALLEL_OnDOSKernelInit));
+	}
 }
 
