@@ -29,6 +29,9 @@
 #include "mapper.h"
 #include "control.h"
 
+/* do not issue CPU-side I/O here -- this code emulates functions that the GDC itself carries out, not on the CPU */
+#include "cpu_io_is_forbidden.h"
+
 #define crtc(blah) vga.crtc.blah
 
 static Bitu read_cga(Bitu /*port*/,Bitu /*iolen*/);
@@ -962,7 +965,7 @@ void HercBlend(bool pressed) {
 	VGA_SetupDrawing(0);
 }
 
-void Herc_Palette(void) {	
+void Herc_Palette(void) {
 	switch (herc_pal) {
 	case MonochromeColor::White:
 		VGA_DAC_SetEntry(0x7,0x2a,0x2a,0x2a);
