@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2002-2020  The DOSBox Team
+ *  Copyright (C) 2002-2021  The DOSBox Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -20,9 +20,15 @@
 #include "mem.h"
 #include "inout.h"
 #include "int10.h"
+#include "render.h"
+#include <map>
 
 #define ACTL_MAX_REG   0x14
 
+#if defined(USE_TTF)
+void GFX_EndTextLines(bool force);
+bool setColors(const char *colorArray, int n);
+#endif
 static INLINE void ResetACTL(void) {
 	IO_Read(real_readw(BIOSMEM_SEG,BIOSMEM_CRTC_ADDRESS) + 6u);
 }
@@ -224,6 +230,18 @@ void INT10_SetSingleDACRegister(uint8_t index,uint8_t red,uint8_t green,uint8_t
 		IO_Write(VGAREG_DAC_DATA,ic);
 		IO_Write(VGAREG_DAC_DATA,ic);
 	}
+#if defined(USE_TTF)
+    char value[30] = "";
+    sprintf(value,"(%d,%d,%d)",red*255/63, green*255/63, blue*255/63);
+    std::map<uint8_t,int> imap;
+    for (uint8_t i = 0; i < 0x10; i++) {
+        ResetACTL();
+        IO_WriteB(VGAREG_ACTL_ADDRESS, i+32);
+        imap[IO_ReadB(VGAREG_ACTL_READ_DATA)]=i;
+    }
+    setColors(value,imap[index]);
+    if (ttf.inUse) GFX_EndTextLines(true);
+#endif
 }
 
 void INT10_GetSingleDACRegister(uint8_t index,uint8_t * red,uint8_t * green,uint8_t * blue) {
@@ -234,18 +252,34 @@ void INT10_GetSingleDACRegister(uint8_t index,uint8_t * red,uint8_t * green,uint
 }
 
 void INT10_SetDACBlock(uint16_t index,uint16_t count,PhysPt data) {
- 	IO_Write(VGAREG_DAC_WRITE_ADDRESS,(uint8_t)index);
+#if defined(USE_TTF)
+    std::map<uint8_t,int> imap;
+	char value[30];
+#endif
+	uint8_t red, green, blue;
+	uint16_t start=count;
+	std::string str="";
+	IO_Write(VGAREG_DAC_WRITE_ADDRESS,(uint8_t)index);
 	if ((real_readb(BIOSMEM_SEG,BIOSMEM_MODESET_CTL)&0x06)==0) {
 		for (;count>0;count--) {
-			IO_Write(VGAREG_DAC_DATA,mem_readb(data++));
-			IO_Write(VGAREG_DAC_DATA,mem_readb(data++));
-			IO_Write(VGAREG_DAC_DATA,mem_readb(data++));
+			red=mem_readb(data++);
+			green=mem_readb(data++);
+			blue=mem_readb(data++);
+			IO_Write(VGAREG_DAC_DATA,red);
+			IO_Write(VGAREG_DAC_DATA,green);
+			IO_Write(VGAREG_DAC_DATA,blue);
+#if defined(USE_TTF)
+            if (start==16) {
+                sprintf(value,"(%d,%d,%d)",red*255/63, green*255/63, blue*255/63);
+                str+=std::string(value)+" ";
+            }
+#endif
 		}
 	} else {
 		for (;count>0;count--) {
-			uint8_t red=mem_readb(data++);
-			uint8_t green=mem_readb(data++);
-			uint8_t blue=mem_readb(data++);
+			red=mem_readb(data++);
+			green=mem_readb(data++);
+			blue=mem_readb(data++);
 
 			/* calculate clamped intensity, taken from VGABIOS */
 			uint32_t i=(( 77u*red + 151u*green + 28u*blue ) + 0x80u) >> 8u;
@@ -253,8 +287,20 @@ void INT10_SetDACBlock(uint16_t index,uint16_t count,PhysPt data) {
 			IO_Write(VGAREG_DAC_DATA,ic);
 			IO_Write(VGAREG_DAC_DATA,ic);
 			IO_Write(VGAREG_DAC_DATA,ic);
+#if defined(USE_TTF)
+            if (start==16) {
+                sprintf(value,"(%d,%d,%d)",red*255/63, green*255/63, blue*255/63);
+                str+=std::string(value);
+            }
+#endif
 		}
 	}
+#if defined(USE_TTF)
+    if (str.size()) {
+        setColors(str.c_str(),-1);
+        if (ttf.inUse) GFX_EndTextLines(true);
+    }
+#endif
 }
 
 void INT10_GetDACBlock(uint16_t index,uint16_t count,PhysPt data) {
@@ -304,8 +350,8 @@ void INT10_GetDACPage(uint8_t* mode,uint8_t* page) {
     /* the operations carried out here blanked the display because of the index (0x10/0x14) without bit 5,
      * write a dummy index with bit 5 to reenable the display. Bugfix for "Blue Force" MS-DOS game.
      *
-     * Note that DOSBox SVN has the same bug without this fix, but appears to work because the AC blanking
-     * doesn't work (2019/12/08). */
+     * Note that DOSBox SVN had the same bug without this fix, but appeared to work because the AC blanking
+     * didn't work. DOSBox SVN has a similar fix as of commit r4297. */
     IO_Write(VGAREG_ACTL_ADDRESS,0x10/*index*/ | 0x20/*display enable*/);
     /* read both, to avoid having to read 0x3CC or BIOS data area regs */
     IO_Read(0x3BA); /* reset flip flop */
@@ -393,18 +439,19 @@ void INT10_SetColorSelect(uint8_t val) {
 		IO_Write(0x3d9,temp);
 	else if (machine == MCH_PCJR) {
 		IO_Read(VGAREG_TDY_RESET); // reset the flipflop
-		switch(vga.mode) {
-		case M_TANDY2:
-			IO_Write(VGAREG_TDY_ADDRESS, 0x11);
-			IO_Write(VGAREG_PCJR_DATA, (val&1)? 0xf:0);
-			break;
-		case M_TANDY4:
+		switch(CurMode->mode) {
+		case 4:
+		case 5:
 			for(uint8_t i = 0x11; i < 0x14; i++) {
 				const uint8_t t4_table[] = {0,2,4,6, 0,3,5,0xf};
 				IO_Write(VGAREG_TDY_ADDRESS, i);
 				IO_Write(VGAREG_PCJR_DATA, t4_table[(i-0x10)+((val&1)? 4:0)]);
 			}
 			break;
+		case 6:
+			IO_Write(VGAREG_TDY_ADDRESS, 0x11);
+			IO_Write(VGAREG_PCJR_DATA, val&1? 0xf:0);
+			break;
 		default:
 			// 16-color modes: always write the same palette
 			for(uint8_t i = 0x11; i < 0x20; i++) {
