@@ -351,6 +351,7 @@ void DoPS2Callback(uint16_t data, int16_t mouseX, int16_t mouseY) {
                 CPU_Push16((uint16_t)(ydiff % 256));
                 CPU_Push16((uint16_t)GetWheel8bit());
                 CPU_Push16((uint16_t)0);
+                break;
             default:   // Standard protocol
                 CPU_Push16((uint16_t)mdat);
                 CPU_Push16((uint16_t)(xdiff % 256));
@@ -386,6 +387,7 @@ Bitu PS2_Handler(void) {
 #define MOUSE_MIDDLE_PRESSED 32
 #define MOUSE_MIDDLE_RELEASED 64
 #define MOUSE_WHEEL_MOVED 128
+#define MOUSE_ABSOLUTE 256
 #define MOUSE_DUMMY 256
 #define MOUSE_DELAY 5.0
 
@@ -860,7 +862,7 @@ const char* Mouse_GetSelected(int x1, int y1, int x2, int y2, int w, int h, uint
 		r2=t;
 	}
 	text[0]=0;
-    uint16_t seg = IS_DOSV?GetTextSeg():0;
+    uint16_t seg = IS_DOSV?GetTextSeg():VGAMEM_CTEXT;
     bool ttfuse = false;
     int ttfcols = 0;
 #if defined(USE_TTF)
@@ -892,8 +894,13 @@ const char* Mouse_GetSelected(int x1, int y1, int x2, int y2, int w, int h, uint
     } else
 #endif
 	for (int i=r1; i<=r2; i++) {
+        uint16_t startlen = len;
         bool lead1 = false, lead2 = false;
-        if ((IS_DOSV && DOSV_CheckCJKVideoMode()) || (IS_J3100 && J3_IsJapanese())) {
+        if ((showdbcs && !ttfuse && !IS_DOSV && isDBCSCP()
+#if defined(USE_TTF)
+            && dbcs_sbcs
+#endif
+            ) || (IS_DOSV && DOSV_CheckCJKVideoMode()) || (IS_J3100 && J3_IsJapanese())) {
             for (int k=0; k<c1; k++) {
                 if (lead1) lead1=false;
                 else if (isKanji1(real_readb(seg,(i*c+k)*2))) lead1=true;
@@ -904,6 +911,7 @@ const char* Mouse_GetSelected(int x1, int y1, int x2, int y2, int w, int h, uint
 				uint16_t address=((i*80)+(ttfuse&&rtl?ttfcols-j-1:j))*2;
 				PhysPt where = CurMode->pstart+address;
 				result=mem_readw(where);
+                if (!result || (result == 0xFE && j && mem_readw(where-2) == 0)) result = 32;
                 if ((result & 0xFF00u) != 0u && (result & 0x7Cu) == 0x08u && (result%0xff) - (result/0x100) == 0xB) {
                     uint8_t val = result/0x100+31;
                     for (auto it = pc98boxdrawmap.begin(); it != pc98boxdrawmap.end(); ++it)
@@ -930,6 +938,7 @@ const char* Mouse_GetSelected(int x1, int y1, int x2, int y2, int w, int h, uint
                 if (lead2) lead2=false;
                 else if (isKanji1(real_readb(seg,(i*c+j)*2))) lead2=true;
                 result=real_readb(seg,(i*c+j)*2);
+                if ((uint8_t)result==0) result=32;
                 text[len++]=result;
                 if (!lead2 && del_flag && (text[len-1]&0xFF) == 0x7F) text[len-1]++;
                 if (j==c2&&c2<c-1&&lead2) {
@@ -945,7 +954,7 @@ const char* Mouse_GetSelected(int x1, int y1, int x2, int y2, int w, int h, uint
                 && showdbcs) ? std::find(jtbs.begin(), jtbs.end(), std::make_pair(i,j)) != jtbs.end():false;
                 if (!isJEGAEnabled()||j>c1||!find) {
                     ReadCharAttr(ttfuse&&rtl?ttfcols-j-1:j,i,page,&result);
-                    if (!result && CurMode->type == M_DCGA && !IS_J3100) result=32;
+                    if ((uint8_t)result==0) result=32;
 #if defined(USE_TTF)
                     if (ttfuse && isDBCSCP()) {
                         ttf_cell *curAC = curAttrChar+i*ttfcols;
@@ -971,11 +980,11 @@ const char* Mouse_GetSelected(int x1, int y1, int x2, int y2, int w, int h, uint
                 }
 			}
 		}
-		while (len>0&&text[len-1]==32) text[--len]=0;
+		if (ttfuse&&rtl) std::reverse(text+startlen, text+len);
+		while (len>0&&text[len-1]==32) {text[--len]=0;bdlist.remove(len);}
 		if (i<r2) text[len++]='\n';
 	}
     text[len] = 0;
-    if (ttfuse&&rtl) std::reverse(text, text+len);
 	*textlen=len;
 	return text;
 }
@@ -1955,7 +1964,8 @@ static Bitu INT74_Handler(void) {
 
         /* Check for an active Interrupt Handler that will get called */
         if (AllowINT33RMAccess() && (mouse.sub_mask & mouse.event_queue[mouse.events].type)) {
-            reg_ax=mouse.event_queue[mouse.events].type;
+            reg_ax=mouse.event_queue[mouse.events].type
+                  | (!MOUSE_IsLocked() ? MOUSE_ABSOLUTE & mouse.sub_mask : 0);
             reg_bl=mouse.event_queue[mouse.events].buttons;
             reg_bh=GetWheel8bit(); /* CuteMouse wheel extension */
             reg_cx=(uint16_t)POS_X;
