@@ -331,7 +331,7 @@ bool Prop_int::CheckValue(Value const& in, bool warn) {
 
 bool Prop_double::SetValue(std::string const& input) {
     Value val;
-    if (!val.SetValue(input,Value::V_DOUBLE)) return false;
+    if (!val.SetValue(input.size()?input:default_value.ToString(),Value::V_DOUBLE)) return false;
     return SetVal(val,false,/*warn*/true);
 }
 
@@ -369,7 +369,7 @@ bool Prop_double::CheckValue(Value const& in, bool warn)
 
 bool Prop_int::SetValue(std::string const& input) {
     Value val;
-    if (!val.SetValue(input,Value::V_INT)) return false;
+    if (!val.SetValue(input.size()?input:default_value.ToString(),Value::V_INT)) return false;
     return SetVal(val,false,/*warn*/true);
 }
 
@@ -428,7 +428,7 @@ bool Prop_bool::SetValue(std::string const& input) {
 
 bool Prop_hex::SetValue(std::string const& input) {
     Value val;
-    if (!val.SetValue(input,Value::V_HEX)) return false;
+    if (!val.SetValue(input.size()?input:default_value.ToString(),Value::V_HEX)) return false;
     return SetVal(val,false,/*warn*/true);
 }
 
@@ -1235,7 +1235,7 @@ bool CommandLine::FindEntry(char const * const name,cmd_it & it,bool neednext) {
 bool CommandLine::FindStringBegin(char const* const begin,std::string & value, bool remove) {
     size_t len = strlen(begin);
     for (cmd_it it = cmds.begin(); it != cmds.end();++it) {
-        if (strncmp(begin,(*it).c_str(),len)==0) {
+        if (strncasecmp(begin,(*it).c_str(),len)==0) {
             value=((*it).c_str() + len);
             if (remove) cmds.erase(it);
             return true;
