@@ -83,6 +83,7 @@ bool initttf = false;
 bool copied = false;
 bool firstset = true;
 bool forceswk = false;
+bool notrysgf = false;
 bool wpExtChar = false;
 int wpType = 0;
 int wpVersion = 0;
@@ -91,6 +92,7 @@ int wpFG = 7;
 int lastset = 0;
 int lastfontsize = 0;
 int switchoutput = -1;
+int checkcol = 0;
 
 static unsigned long ttfSize = sizeof(DOSBoxTTFbi), ttfSizeb = 0, ttfSizei = 0, ttfSizebi = 0;
 static void * ttfFont = DOSBoxTTFbi, * ttfFontb = NULL, * ttfFonti = NULL, * ttfFontbi = NULL;
@@ -108,12 +110,6 @@ static SDL_Rect ttf_textClip = {0, 0, 0, 0};
 ttf_cell curAttrChar[txtMaxLins*txtMaxCols];					// currently displayed textpage
 ttf_cell newAttrChar[txtMaxLins*txtMaxCols];					// to be replaced by
 
-typedef struct {
-	uint8_t red;
-	uint8_t green;
-	uint8_t blue;
-	uint8_t alpha;		// unused
-} alt_rgb;
 alt_rgb altBGR0[16], altBGR1[16];
 int blinkCursor = -1;
 static int prev_sline = -1;
@@ -123,7 +119,7 @@ static bool blinkstate = false;
 bool colorChanged = false, justChanged = false, staycolors = false, firstsize = true, ttfswitch=false, switch_output_from_ttf=false;
 
 int menuwidth_atleast(int width), FileDirExistCP(const char *name), FileDirExistUTF8(std::string &localname, const char *name);
-void AdjustIMEFontSize(void), initcodepagefont(void), change_output(int output), drawmenu(Bitu val), MSG_Init(void), KEYBOARD_Clear(void), RENDER_Reset(void), DOSBox_SetSysMenu(void), GetMaxWidthHeight(unsigned int *pmaxWidth, unsigned int *pmaxHeight), SetWindowTransparency(int trans), resetFontSize(void), RENDER_CallBack( GFX_CallBackFunctions_t function );
+void AdjustIMEFontSize(void),refreshExtChar(void), initcodepagefont(void), change_output(int output), drawmenu(Bitu val), KEYBOARD_Clear(void), RENDER_Reset(void), DOSBox_SetSysMenu(void), GetMaxWidthHeight(unsigned int *pmaxWidth, unsigned int *pmaxHeight), SetWindowTransparency(int trans), resetFontSize(void), RENDER_CallBack( GFX_CallBackFunctions_t function );
 bool isDBCSCP(void), InitCodePage(void), CodePageGuestToHostUTF16(uint16_t *d/*CROSS_LEN*/,const char *s/*CROSS_LEN*/), systemmessagebox(char const * aTitle, char const * aMessage, char const * aDialogType, char const * aIconType, int aDefaultButton);
 std::string GetDOSBoxXPath(bool withexe=false);
 
@@ -244,9 +240,9 @@ void setVGADAC() {
             IO_WriteB(VGAREG_ACTL_ADDRESS, i+32);
             imap[i]=IO_ReadB(VGAREG_ACTL_READ_DATA);
             IO_WriteB(VGAREG_DAC_WRITE_ADDRESS, imap[i]);
-            IO_WriteB(VGAREG_DAC_DATA, altBGR1[i].red*63/255);
-            IO_WriteB(VGAREG_DAC_DATA, altBGR1[i].green*63/255);
-            IO_WriteB(VGAREG_DAC_DATA, altBGR1[i].blue*63/255);
+            IO_WriteB(VGAREG_DAC_DATA, (altBGR1[i].red+3)*63/255);
+            IO_WriteB(VGAREG_DAC_DATA, (altBGR1[i].green+3)*63/255);
+            IO_WriteB(VGAREG_DAC_DATA, (altBGR1[i].blue+3)*63/255);
         }
     }
 }
@@ -493,6 +489,43 @@ void CheckTTFLimit() {
     }
 }
 
+int setTTFMap(bool changecp) {
+    char text[2];
+    uint16_t uname[4], wcTest[256];
+    int cp = dos.loaded_codepage;
+    for (int i = 0; i < 256; i++) {
+        text[0]=i;
+        text[1]=0;
+        uname[0]=0;
+        uname[1]=0;
+        if (cp == 932 && (halfwidthkana || IS_JEGA_ARCH)) forceswk=true;
+        if (cp == 932 || cp == 936 || cp == 949 || cp == 950 || cp == 951) dos.loaded_codepage = 437;
+        if (CodePageGuestToHostUTF16(uname,text)) {
+            wcTest[i] = uname[1]==0?uname[0]:i;
+            if (cp == 932 && lowboxdrawmap.find(i)!=lowboxdrawmap.end() && TTF_GlyphIsProvided(ttf.SDL_font, wcTest[i]))
+                cpMap[i] = wcTest[i];
+        }
+        forceswk=false;
+        if (cp == 932 || cp == 936 || cp == 949 || cp == 950 || cp == 951) dos.loaded_codepage = cp;
+    }
+    uint16_t unimap;
+    int notMapped = 0;
+    for (int y = ((cpMap[1]!=1&&cpMap[1]!=0x263A)||cp==867||(customcp&&(dos.loaded_codepage==customcp||changecp))||(altcp&&(dos.loaded_codepage==altcp||changecp))?0:8); y < 16; y++)
+        for (int x = 0; x < 16; x++) {
+            if (y<8 && (wcTest[y*16+x] == y*16+x || wcTest[y*16+x] == cp437_to_unicode[y*16+x])) unimap = cpMap_copy[y*16+x];
+            else unimap = wcTest[y*16+x];
+            if (!TTF_GlyphIsProvided(ttf.SDL_font, unimap)) {
+                cpMap[y*16+x] = 0;
+                notMapped++;
+                LOG_MSG("Unmapped character: %3d - %4x", y*16+x, unimap);
+            } else
+                cpMap[y*16+x] = unimap;
+        }
+    if (eurAscii != -1 && TTF_GlyphIsProvided(ttf.SDL_font, 0x20ac))
+        cpMap[eurAscii] = 0x20ac;
+    return notMapped;
+}
+
 int setTTFCodePage() {
     if (!copied) {
         memcpy(cpMap_copy,cpMap,sizeof(cpMap[0])*256);
@@ -508,38 +541,7 @@ int setTTFCodePage() {
 
     if (cp) {
         LOG_MSG("Loaded system codepage: %d\n", cp);
-        char text[2];
-        uint16_t uname[4], wcTest[256];
-        for (int i = 0; i < 256; i++) {
-            text[0]=i;
-            text[1]=0;
-            uname[0]=0;
-            uname[1]=0;
-            if (cp == 932 && (halfwidthkana || IS_JEGA_ARCH)) forceswk=true;
-            if (cp == 932 || cp == 936 || cp == 949 || cp == 950 || cp == 951) dos.loaded_codepage = 437;
-            if (CodePageGuestToHostUTF16(uname,text)) {
-                wcTest[i] = uname[1]==0?uname[0]:i;
-                if (cp == 932 && lowboxdrawmap.find(i)!=lowboxdrawmap.end() && TTF_GlyphIsProvided(ttf.SDL_font, wcTest[i]))
-                    cpMap[i] = wcTest[i];
-            }
-            forceswk=false;
-            if (cp == 932 || cp == 936 || cp == 949 || cp == 950 || cp == 951) dos.loaded_codepage = cp;
-        }
-        uint16_t unimap;
-        int notMapped = 0;
-        for (int y = ((customcp&&dos.loaded_codepage==customcp)||(altcp&&dos.loaded_codepage==altcp)?0:8); y < 16; y++)
-            for (int x = 0; x < 16; x++) {
-                if (y<8 && (wcTest[y*16+x] == y*16+x || wcTest[y*16+x] == cp437_to_unicode[y*16+x])) unimap = cpMap_copy[y*16+x];
-                else unimap = wcTest[y*16+x];
-                if (!TTF_GlyphIsProvided(ttf.SDL_font, unimap)) {
-                    cpMap[y*16+x] = 0;
-                    notMapped++;
-                    LOG_MSG("Unmapped character: %3d - %4x", y*16+x, unimap);
-                } else
-                    cpMap[y*16+x] = unimap;
-            }
-        if (eurAscii != -1 && TTF_GlyphIsProvided(ttf.SDL_font, 0x20ac))
-            cpMap[eurAscii] = 0x20ac;
+        int notMapped = setTTFMap(true);
         if (strcmp(RunningProgram, "LOADLIN") && !dos_kernel_disabled)
             initcodepagefont();
 #if defined(WIN32) && !defined(HX_DOS)
@@ -547,9 +549,7 @@ int setTTFCodePage() {
 #endif
         if(IS_JEGA_ARCH) memcpy(cpMap,cpMap_AX,sizeof(cpMap[0])*32);
         if (cp == 932 && halfwidthkana) resetFontSize();
-        if (cp == 936) mainMenu.get_item("ttf_extcharset").check(gbk).refresh_item(mainMenu);
-        else if (cp == 950 || cp == 951) mainMenu.get_item("ttf_extcharset").check(chinasea).refresh_item(mainMenu);
-        else mainMenu.get_item("ttf_extcharset").check(gbk&&chinasea).refresh_item(mainMenu);
+        refreshExtChar();
         return notMapped;
     } else
         return -1;
@@ -641,7 +641,7 @@ void OUTPUT_TTF_Select(int fsize) {
         bool trysgf = false;
         if (!*fName) {
             std::string mtype(static_cast<Section_prop *>(control->GetSection("dosbox"))->Get_string("machine"));
-            if (IS_PC98_ARCH||mtype.substr(0, 4)=="pc98"||(InitCodePage()&&isDBCSCP())) trysgf = true;
+            if (IS_PC98_ARCH||mtype.substr(0, 4)=="pc98"||(!notrysgf&&InitCodePage()&&isDBCSCP())) trysgf = true;
         }
         force_conversion = false;
         dos.loaded_codepage = cp;
@@ -1107,7 +1107,7 @@ void GFX_EndTextLines(bool force) {
                     if(ttf_dosv && ascii == 0x5c)
                         ascii = 0x9d;
                     curAC[x] = newAC[x];
-                    if (ascii > 175 && ascii < 179 && !IS_PC98_ARCH && !IS_JEGA_ARCH && dos.loaded_codepage != 864 && dos.loaded_codepage != 874 && dos.loaded_codepage != 3021 && !(dos.loaded_codepage == 932 && halfwidthkana) && (dos.loaded_codepage < 1250 || dos.loaded_codepage > 1258) && !(altcp && dos.loaded_codepage == altcp) && !(customcp && dos.loaded_codepage == customcp)) {	// special: shade characters 176-178 unless PC-98
+                    if (ascii > 175 && ascii < 179 && !IS_PC98_ARCH && !IS_JEGA_ARCH && dos.loaded_codepage != 864 && dos.loaded_codepage != 868 && dos.loaded_codepage != 874 && dos.loaded_codepage != 3021 && !(dos.loaded_codepage == 932 && halfwidthkana) && (dos.loaded_codepage < 1250 || dos.loaded_codepage > 1258) && !(altcp && dos.loaded_codepage == altcp) && !(customcp && dos.loaded_codepage == customcp)) {	// special: shade characters 176-178 unless PC-98
                         ttf_bgColor.b = (ttf_bgColor.b*(179-ascii) + ttf_fgColor.b*(ascii-175))>>2;
                         ttf_bgColor.g = (ttf_bgColor.g*(179-ascii) + ttf_fgColor.g*(ascii-175))>>2;
                         ttf_bgColor.r = (ttf_bgColor.r*(179-ascii) + ttf_fgColor.r*(ascii-175))>>2;
@@ -1374,7 +1374,8 @@ void ttf_setlines(int cols, int lins) {
 
 void ttf_switch_on(bool ss=true) {
     if ((ss&&ttfswitch)||(!ss&&switch_output_from_ttf)) {
-        if (strcmp(RunningProgram, "LOADLIN") && !dos_kernel_disabled) {
+        checkcol = 0;
+        if (strcmp(RunningProgram, "LOADLIN")) {
             uint16_t oldax=reg_ax;
             reg_ax=0x1600;
             CALLBACK_RunRealInt(0x2F);
@@ -1415,10 +1416,12 @@ void ttf_switch_on(bool ss=true) {
         transparency = 0;
         SetWindowTransparency(static_cast<Section_prop *>(control->GetSection("sdl"))->Get_int("transparency"));
 #endif
+        if (!IS_PC98_ARCH && vga.draw.address_add != ttf.cols * 2) checkcol = ss?2:1;
     }
 }
 
 void ttf_switch_off(bool ss=true) {
+    checkcol = 0;
     if (!ss&&ttfswitch)
         ttf_switch_on();
     if (ttf.inUse) {
@@ -1465,6 +1468,7 @@ void ttf_switch_off(bool ss=true) {
         transparency = 0;
         SetWindowTransparency(static_cast<Section_prop *>(control->GetSection("sdl"))->Get_int("transparency"));
 #endif
+        if (Mouse_IsLocked()) GFX_CaptureMouse(true);
     }
 }
 #endif
