@@ -75,10 +75,11 @@ emptyline:
 				//So we continue reading till EOL/EOF
 				if (((cmd_write - temp) + 1) < (CMD_MAXLINE - 1))
 					*cmd_write++ = (char)c;
-			} else if (c!=0x1a) {
-				if (c != '\n' && c != '\r')
-					LOG(LOG_MISC,LOG_DEBUG)("Encountered non-standard control character in batch file: Dec %03u and Hex %#04x.\n", c, c);
-			}
+			} else if (c==0x1a) {
+				n = 0;
+				break;
+			} else if (c != '\n' && c != '\r')
+				LOG(LOG_MISC,LOG_DEBUG)("Encountered non-standard control character in batch file: Dec %03u and Hex %#04x.\n", c, c);
 		}
 	} while (c!='\n' && n);
 	*cmd_write=0;
@@ -202,7 +203,10 @@ again:
 			if (c>31) {
 				if (((cmd_write - cmd_buffer) + 1) < (CMD_MAXLINE - 1))
 					*cmd_write++ = (char)c;
-			} else if (c!=0x1a && c!=0x1b && c!='\t' && c!=7 && c!=8) {
+			} else if (c==0x1a) {
+				n = 0;
+				break;
+			} else if (c!=0x1b && c!='\t' && c!=7 && c!=8) {
 					if (c != '\n' && c != '\r')
 					LOG(LOG_MISC,LOG_DEBUG)("Encountered non-standard control character in batch file: Dec %03u and Hex %#04x.\n", c, c);
 			}
