@@ -36,6 +36,8 @@ cat >>include/SDL_config.h <<_EOF
 #define SDL_WIN32_HX_DOS
 #endif
 _EOF
+
+echo "#undef ENABLE_IM_EVENT" >>include/SDL_platform.h
 fi
 
 # SDL is having concurrency problems with Brew compiles, help it out
@@ -51,4 +53,5 @@ make install || exit 1  # will install into ./linux-host
 # STOP DELETING THE FILE!!!!! I need it for Windows builds!
 cd "$srcdir" || exit 1
 cp -v include/SDL_config.h.default include/SDL_config.h
+cp -v include/SDL_platform.h.default include/SDL_platform.h
 
